#!/bin/bash
#
# Create CA Intermediate
# 
# 
# This function will generate a CA Intermediate
# IN: UNIQ_ID_CA, SERIAL
#
PARAM1=$1
PARAM2=$2

usage() {
  echo
  echo "Generate a new CA Intermediate certificate"
  echo
  echo "This program will generate a new certificate authority (CA) intermediate"
  echo "It requires a CA certificate used to sign CA Intermediate"
  echo "Requires the file \"ca.pem\" that is used to sign the certificates"
  echo ""
  echo ""
  echo ""
  echo
  echo "  usage:   gen_ca-i.sh  <Org URL> <Serial>"
  echo
  echo "  example: gen_ca-i.sh  skunkworks.acme.xyz"
  echo "                        10052"
  echo
  exit 1
}

error_no_ca_file() {
  echo
  echo "ERROR: missing ca.pem"
  echo
  usage
}


generate_ca_i() {
  echo_block "Create CA Intermediate  (${UNIQ_ID_CA})"
  # params
  UNIQ_ID_CA=$1
  SERIAL=$2

  openssl genrsa -out "ca_i_${UNIQ_ID_CA}.keys.pem" 4096

  # Create Cert Signing Request (CSR)
  openssl req -config $CA_CNF -new -sha256 \
              -subj "/C=OO/O=ACME/OU=ACME Intermediate/CN=${UNIQ_ID_CA}" \
              -key "ca_i_${UNIQ_ID_CA}.keys.pem" -out "ca_i_${UNIQ_ID_CA}.csr.pem"

  # Create Certificate   (valid for ~2 years, after the entire chain of trust expires)
  # CA signs Intermediate
  openssl x509 -req -days 750 -extfile $CA_CNF -extensions v3_ca_i \
               -CA $FQ_CA_CERT -CAkey $FQ_CA_KEYS -set_serial ${SERIAL} \
               -in "ca_i_${UNIQ_ID_CA}.csr.pem" -out "ca_i_${UNIQ_ID_CA}.crt.pem"

  # Package the Certificate Authority Certificates for distro  (windoze needs this)
  openssl pkcs12 -export -password "pass:password" -inkey "ca_i_${UNIQ_ID_CA}.keys.pem" \
                 -name "CA Intermediate Mobile Provision" -certfile $FQ_CA_CERT \
                 -in "ca_i_${UNIQ_ID_CA}.crt.pem" -out "ca_i_${UNIQ_ID_CA}.p12"

  # verify certificate (output to text file for review)
  openssl x509 -noout -text -in "ca_i_${UNIQ_ID_CA}.crt.pem" > "ca_i_${UNIQ_ID_CA}_crt_info.txt"

  # create certifiate chain
  cat $FQ_CA_CERT "ca_i_${UNIQ_ID_CA}.crt.pem" > "ca_cert-chain_${UNIQ_ID_CA}.crts.pem"
}


main() {
  if [[ ! -f ca-i.pem ]]; then
    error_no_ca_file
  fi

  if [[ -n $PARAM1 ]] || [[ -n $PARAM2 ]]; then
    generate_ca_i $PARAM1 $PARAM2
  else
    usage
  fi
}

main
